import logging
import click
from dotenv import load_dotenv
import os

from udata_analysis_service.background_tasks import celery
from udata_event_service.consumer import consume_kafka

from udata_analysis_service.consumer import process_message
from udata_analysis_service.utils.kafka import get_topic


@click.group()
@click.version_option()
def cli() -> None:
    """
    udata-analysis-service
    """


@cli.command()
def consume() -> None:
    """Launch Kafka consumer loop"""
    load_dotenv()
    logging.basicConfig(level=logging.INFO)
    consume_kafka(
        f"{os.environ['KAFKA_HOST']}:{os.environ['KAFKA_PORT']}",
        group_id=None,
        topics=get_topic("resource.stored"),
        message_processing_func=process_message,
    )


@cli.command()
def work() -> None:
    """Starts a worker"""
    worker = celery.Worker()
    worker.start()
    return worker.exitcode



import logging
import os

import boto3
from botocore.client import Config, ClientError
from celery import Celery
from dotenv import load_dotenv

from csv_detective.explore_csv import routine_minio
from udata_event_service.producer import produce
from udata_analysis_service.utils.kafka import get_topic

load_dotenv()

ROWS_TO_ANALYSE_PER_FILE = int(
    os.environ.get("ROWS_TO_ANALYSE_PER_FILE", "500")
)
BROKER_URL = os.environ.get("BROKER_URL", "redis://localhost:6381/0")
celery = Celery("tasks", broker=BROKER_URL)


@celery.task
def manage_resource(
    dataset_id: str,
    resource_id: str,
    resource_location: dict,
    minio_user: str,
    minio_pwd: str,
) -> None:
    logging.info(
        "Processing task for resource {} in dataset {}".format(
            resource_id, dataset_id
        )
    )

    # Ensure credentials are correct and bucket exists
    s3_client = boto3.client(
        "s3",
        endpoint_url=resource_location["netloc"],
        aws_access_key_id=minio_user,
        aws_secret_access_key=minio_pwd,
        config=Config(signature_version="s3v4"),
    )
    try:
        s3_client.head_bucket(Bucket=resource_location["bucket"])
    except ClientError as e:
        logging.error(e)
        logging.error(
            "Bucket {} does not exist or credentials are invalid".format(
                resource_location["bucket"]
            )
        )
        return

    resource_key = resource_location["key"]
    csv_minio_location = {
        "url": resource_location["netloc"],
        "bucket": resource_location["bucket"],
        "key": resource_key,
    }
    output_minio_location = {
        "url": resource_location["netloc"],
        "bucket": os.environ["CSV_DETECTIVE_REPORT_BUCKET"],
        "key": os.path.join(
            os.environ["CSV_DETECTIVE_REPORT_FOLDER"], dataset_id, resource_id
        )
        + ".json",
    }
    tableschema_minio_location = {
        "url": resource_location["netloc"],
        "bucket": os.environ["TABLESCHEMA_BUCKET"],
        "key": os.path.join(
            os.environ["TABLESCHEMA_FOLDER"],
            dataset_id,
            resource_id,
        ),
    }

    routine_minio(
        csv_minio_location=csv_minio_location,
        output_minio_location=output_minio_location,
        tableschema_minio_location=tableschema_minio_location,
        minio_user=minio_user,
        minio_pwd=minio_pwd,
        num_rows=ROWS_TO_ANALYSE_PER_FILE,
        user_input_tests="ALL",
    )
    produce(
        f"{os.environ['KAFKA_HOST']}:{os.environ['KAFKA_PORT']}",
        get_topic("resource.analysed"),
        service="csvdetective",
        key_id=resource_id,
        document={"location": output_minio_location},
        meta={"dataset_id": dataset_id, "message_type": "resource.analysed"},
    )



import os
from unittest import mock

import pytest

from udata_analysis_service.utils.kafka import get_topic

@mock.patch.dict(os.environ, {'UDATA_INSTANCE_NAME': 'udata'})
def test_get_topic():
    topic = get_topic('test')
    assert topic == 'udata.test'



import logging
import os

from udata_analysis_service.background_tasks import manage_resource


def process_message(resource_id: str, message: dict, topic: str) -> None:
    logging.info(f"New message detected, topic {topic}, checking resource {resource_id}")

    dataset_id = message["meta"]["dataset_id"]
    manage_resource.delay(
        dataset_id,
        resource_id,
        message["value"]["location"],
        minio_user=os.environ["MINIO_USER"],
        minio_pwd=os.environ["MINIO_PWD"],
    )



import os

import csv_detective
import pytest

from udata_analysis_service.background_tasks import manage_resource

pytestmark = pytest.mark.asyncio


async def test_manage_resource_send_produce_message(mocker):
    mocker.patch.dict(os.environ, {
        'CSV_DETECTIVE_REPORT_BUCKET': 'detective-bucket',
        'CSV_DETECTIVE_REPORT_FOLDER': 'report',
        'TABLESCHEMA_BUCKET': '',
        'TABLESCHEMA_FOLDER': '',
        'KAFKA_HOST': 'localhost',
        'KAFKA_PORT': '9092',
        'UDATA_INSTANCE_NAME': 'udata'
    })
    mocker.patch('boto3.client')
    routine = mocker.patch('udata_analysis_service.background_tasks.routine_minio')
    produce = mocker.patch('udata_analysis_service.background_tasks.produce')

    manage_resource(
        dataset_id = 'dataset_id',
        resource_id = 'resource_id',
        resource_location = {'netloc': 'netloc', 'bucket': 'bucket', 'key': 'key'},
        minio_user = 'minio_user',
        minio_pwd = 'minio_pwd',
    )

    produce.assert_called_with(
        'localhost:9092',
        'udata.resource.analysed',
        service='csvdetective',
        key_id='resource_id',
        document={
            'location': {
                'url': 'netloc',
                'bucket': 'detective-bucket',
                'key': 'report/dataset_id/resource_id.json'
            }
        },
        meta={'dataset_id': 'dataset_id', 'message_type': 'resource.analysed'},
    )



"""
udata analysis service
"""


import os

tag = os.environ.get('CIRCLE_TAG')
build_num = os.environ.get('CIRCLE_BUILD_NUM')

__version__ = '0.0.1.dev' + (str(build_num) if not tag and build_num else '')



import os


def get_topic(topic_suffix: str) -> str:
    """Returns topic name based on topic suffix by preprending '{instance_name}.'
    eg: 'resource.analysed' -> 'udata.resource.analysed'
    """
    return f"{os.environ['UDATA_INSTANCE_NAME']}.{topic_suffix}"
